UNIT evaluation;

INTERFACE

USES population, definitions;

PROCEDURE pop_evaluation(pop_size        : pop_size_type;
                         VAR pop_ev_table: pop_ev_table_type;
                         VAR pop         : pop_type);

PROCEDURE fitness_is_evaluation(in_table     : pop_ev_table_type;
                                pop_size     : integer;
                                VAR out_table: pop_ev_table_type);

PROCEDURE linear_normalisation(in_table               : pop_ev_table_type;
                               pop_size, best_fitness,
                               decrement              : integer;
                               VAR out_table          : pop_ev_table_type);

PROCEDURE windowing(in_table       : pop_ev_table_type;
                    guard, pop_size: integer;
                    VAR out_table  : pop_ev_table_type);

IMPLEMENTATION

FUNCTION distance(city_1, city_2: char):integer;

    VAR other : char;


    PROCEDURE london_calc;
    BEGIN
      IF city_1 = 'L' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'A': distance := 220;
        'B': distance := 580;
        'C': distance := 600;
        'M': distance := 1580;
        'O': distance := 720;
        'P': distance := 200;
        'R': distance := 880;
        'V': distance := 700;
        'W': distance := 1000;
        'Z': distance := 480;
       END;
    END;

    PROCEDURE amsterdam_calc;
    BEGIN
      IF city_1 = 'A' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'B': distance := 360;
        'C': distance := 380;
        'M': distance := 1300;
        'O': distance := 560;
        'P': distance := 820;
        'R': distance := 800;
        'V': distance := 580;
        'W': distance := 660;
        'Z': distance := 380;
       END;
    END;

    PROCEDURE berlin_calc;
    BEGIN
      IF city_1 = 'B' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'C': distance := 220;
        'M': distance := 1040;
        'O': distance := 520;
        'P': distance := 540;
        'R': distance := 720;
        'V': distance := 480;
        'W': distance := 360;
        'Z': distance := 420;
       END;
    END;

    PROCEDURE copenhagen_calc;
    BEGIN
      IF city_1 = 'C' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'M': distance := 1000;
        'O': distance := 300;
        'P': distance := 640;
        'R': distance := 940;
        'V': distance := 700;
        'W': distance := 460;
        'Z': distance := 600;
       END;
    END;

    PROCEDURE moscow_calc;
    BEGIN
      IF city_1 = 'M' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'O': distance := 960;
        'P': distance := 1580;
        'R': distance := 1500;
        'V': distance := 1340;
        'W': distance := 720;
        'Z': distance := 1400;
       END;
    END;

    PROCEDURE oslo_calc;
    BEGIN
      IF city_1 = 'O' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'P': distance := 820;
        'R': distance := 1220;
        'V': distance := 980;
        'W': distance := 620;
        'Z': distance := 860;
       END;
    END;


    PROCEDURE paris_calc;
    BEGIN
      IF city_1 = 'P' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'R': distance := 680;
        'V': distance := 520;
        'W': distance := 880;
        'Z': distance := 300;
       END;
    END;

    PROCEDURE rome_calc;
    BEGIN
      IF city_1 = 'R' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'V': distance := 240;
        'W': distance := 840;
        'Z': distance := 440;
       END;
    END;

    PROCEDURE venice_calc;
    BEGIN
      IF city_1 = 'V' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'W': distance := 640;
        'Z': distance := 220;
       END;
    END;

    PROCEDURE warsaw_calc;
    BEGIN
      IF city_1 = 'W' THEN other := city_2 ELSE other := city_1;
       CASE other OF
        'Z': distance := 680;
       END;
    END;

BEGIN
  IF ((city_1 = 'L') OR (city_2 = 'L')) THEN london_calc
   ELSE IF ((city_1 = 'A') OR (city_2 = 'A')) THEN amsterdam_calc
    ELSE IF ((city_1 = 'B') OR (city_2 = 'B')) THEN berlin_calc
     ELSE IF ((city_1 = 'C') OR (city_2 = 'C')) THEN copenhagen_calc
      ELSE IF ((city_1 = 'M') OR (city_2 = 'M')) THEN moscow_calc
       ELSE IF ((city_1 = 'O') OR (city_2 = 'O')) THEN oslo_calc
        ELSE IF ((city_1 = 'P') OR (city_2 = 'P')) THEN paris_calc
         ELSE IF ((city_1 = 'R') OR (city_2 = 'R')) THEN rome_calc
          ELSE IF ((city_1 = 'V') OR (city_2 = 'V')) THEN venice_calc
           ELSE IF ((city_1 = 'W') OR (city_2 = 'W')) THEN warsaw_calc;
END;


FUNCTION chromo_evaluation(chromo: chromo_type): integer;

    VAR index, evaluation: integer;
        city_1, city_2   : char;

BEGIN
  evaluation := 0;
  FOR index := 1 TO (chromo_length - 1) DO
  BEGIN
    city_1 := chromo[index];
    city_2 := chromo[index+1];
    evaluation := evaluation + distance(city_1, city_2);
  END;
  chromo_evaluation:= evaluation
END;


PROCEDURE pop_evaluation(pop_size        : pop_size_type;
                         VAR pop_ev_table: pop_ev_table_type;
                         VAR pop         : pop_type);

    VAR chromo_no      : integer;
        temp_eval_store: eval_store;

BEGIN
  FOR chromo_no := 1 TO pop_size DO
  BEGIN
    temp_eval_store[1] := chromo_evaluation(pop[chromo_no]);
    pop_ev_table[chromo_no] := temp_eval_store;
  END;
  quicksort(1, pop_size, pop_ev_table, pop);
END;


PROCEDURE fitness_is_evaluation(in_table     : pop_ev_table_type;
                                pop_size     : integer;
                                VAR out_table: pop_ev_table_type);

    VAR index          : integer;
        running_total  : longint;
        temp_eval_store: eval_store;

BEGIN
  running_total := 0;
  FOR index := 1 TO pop_size DO
  BEGIN
    temp_eval_store := in_table[index];
    temp_eval_store[2] := (14820 - temp_eval_store[1]);
    running_total := running_total + (temp_eval_store[2] DIV 256);
    temp_eval_store[3] := running_total;
    out_table[index] := temp_eval_store;
  END;
END;


PROCEDURE linear_normalisation(in_table               : pop_ev_table_type;
                               pop_size, best_fitness,
                                decrement             : integer;
                               VAR out_table          : pop_ev_table_type);

    VAR index, fitness: integer;
    running_total     : longint;

BEGIN
  running_total := 0;
  fitness := best_fitness;
  FOR index := 1 TO pop_size DO
  BEGIN
    out_table[index, 1] := in_table[index, 1];
    out_table[index, 2] := fitness;
    running_total := running_total + fitness DIV 64;
    out_table[index, 3] := running_total;
    fitness := fitness - decrement;
  END;
END;


PROCEDURE windowing(in_table       : pop_ev_table_type;
                    guard, pop_size: integer;
                    VAR out_table  : pop_ev_table_type);

    VAR index, min_eval: integer;
        running_total  : longint;
        temp_eval_store: eval_store;

BEGIN
  running_total := 0;
  min_eval := 14820 - in_table[pop_size, 1];
  FOR index := 1 TO pop_size DO
  BEGIN
    temp_eval_store := in_table[index];
    temp_eval_store[2] := ((14820 - temp_eval_store[1]) - min_eval);
    IF temp_eval_store[2] < guard THEN temp_eval_store[2] := guard;
    running_total := running_total + (temp_eval_store[2] DIV 256);
    temp_eval_store[3] := running_total;
    out_table[index] := temp_eval_store;
  END;
END;

END.