UNIT HCI;

INTERFACE

USES definitions, crt, printer, graph;

PROCEDURE configure_screen(paper, ink, speed: integer);

PROCEDURE set_window(position: integer);

PROCEDURE write_title(mode: string);

PROCEDURE opening_screens(VAR setup: setup_type);

PROCEDURE fast_setup(VAR setup: setup_type);

PROCEDURE guided_setup(VAR setup: setup_type);

PROCEDURE write_setup(VAR setup: setup_type);

PROCEDURE run_screen;

PROCEDURE show_and_store(results       : char;
                         pop_rec       : pop_rec_type;
                         pop_size,
                         gen_no        : integer; print_mode:
char;
                         VAR best_store: best_store_type;
                         VAR best_five : best_five_type);

PROCEDURE draw_route(chromo: chromo_type; colour: integer);

PROCEDURE draw_bar(last_eval, eval: integer; results: char);

IMPLEMENTATION

PROCEDURE configure_screen(paper, ink , speed: integer);

    VAR count: integer;

BEGIN
  textcolor(ink);
  textbackground(paper);
  count := 13;
  REPEAT
    window(40 - ((14 - count) * 3), count - 1, 40 + ((14 -
count) * 3),
           25-count);
    clrscr;
    count := count - 1;
    IF speed = 2 THEN delay(35)
  UNTIL count = 1;
  window(1, 1, 80, 25);
  clrscr
END;

PROCEDURE set_window(position: integer);
BEGIN
  CASE position OF
   top   : BEGIN
             textbackground(blue);
             textcolor(yellow);
             window(15, 2, 65, 3)
           END;
   middle: BEGIN
             textbackground(blue);
             textcolor(white);
             window(5, 5, 75, 18)
           END;
   bottom: BEGIN
             textbackground(lightgray);
             textcolor(black);
             window(12, 20, 68, 23)
           END;
  END;
  clrscr
END;

PROCEDURE write_title(mode: string);
BEGIN
  set_window(top);
  writeln(' Demonstration Of Using Genetic Algorithm Learning');
  write(mode)
END;

PROCEDURE opening_screens(VAR setup: setup_type);

BEGIN
  set_window(middle);
  clrscr;
  writeln;
  writeln('                             Configuration');
  writeln;
  writeln(' Initially :- ');
  writeln;
  writeln(' Population size = 100 ');
  writeln(' No. of Generations = 50');
  writeln(' Results Format = Pure Results');
  writeln(' Print Mode = Screen only');
  writeln;
  write(' By choosing the "y" option, it is possible to alter ');
  writeln('these settings.');
  REPEAT
    set_window(bottom);
    ClrScr;
    writeln(' Do you wish to alter configuration');
    writeln(' y = yes, n = no');
    writeln(' It is recommended to leave initially');
    write(' >');
    read(setup.config)
  UNTIL (setup.config = 'y') OR (setup.config = 'n');
  IF setup.config = 'n' THEN
  BEGIN
    setup.pop_size := 100;
    setup.gen := 50;
    setup.results := 'p';
    setup.print_mode := 's';
  END;
  set_window(middle);
  clrscr;
  writeln;
  writeln('                               Setup Mode');
  writeln;
  write('  After you have had some experience in using the system, ');
  writeln('you may ');
  write(' prefer to set the system parameters without menu assistance. ');
  writeln(' This');
  writeln(' option allows the choice of which setup route to take.');
  writeln(' ');
  writeln(' ');
  REPEAT
    set_window(bottom);
    ClrScr;
    writeln(' Enter required Setup Mode');
    writeln(' g = guided setup, f = fast setup');
    writeln(' guided setup is recommended initially ');
    write(' >');
    read(setup.set_mode)
  UNTIL (setup.set_mode = 'g') OR (setup.set_mode = 'f')
END;


PROCEDURE fast_setup(VAR setup: setup_type);

BEGIN
  set_window(middle);
  ClrScr;
  writeln;
  writeln('                              Fast Setup');
  set_window(bottom);
  ClrScr;
  IF setup.config = 'y' THEN
  BEGIN
    REPEAT
      write('Population size (multiples of 10, max = 200) ?');
      readln(setup.pop_size);
    UNTIL setup.pop_size MOD 10 = 0;
    write('no of generations (max = 200) ?');
    readln (setup.gen);
    REPEAT
      write('results_format (p/d) ?');
      readln(setup.results);
    UNTIL (setup.results = 'p') or (setup.results = 'd');
    REPEAT
      write('Output to (s/p) ?');
      readln(setup.print_mode);
    UNTIL (setup.print_mode = 'p') OR (setup.print_mode = 's');
  END;
  write('percentage crossover ?');
  readln (setup.cross_percent);
  write('percentage mutation (x10)?');
  readln (setup.mut_percent);
  write('deletion technique (a/s) ?');
  REPEAT
    readln (setup.del);
  UNTIL (setup.del = 'a') OR (setup.del = 's');
  IF setup.del = 'a' THEN
  BEGIN
    REPEAT
      write('elitism value (multiples of 2) ?');
      readln(setup.elite);
    UNTIL setup.elite MOD 2 = 0;
  END;
  REPEAT
    write('fitness measure (e/w/l) ?');
    readln (setup.fitness);
  UNTIL (setup.fitness = 'e') OR (setup.fitness = 'l')
   OR (setup.fitness = 'w');
  IF (setup.fitness = 'w') THEN
  BEGIN
    write('guard (0-4580) ?');
    readln(setup.guard);
  END;
  IF (setup.fitness = 'l') THEN
  BEGIN
    write('decrement (0-9) ?');
    readln(setup.dec);
    write('best fitness (min ',(setup.dec*setup.pop_size),', max 10000) ?');
    readln(setup.best_fitness);
  END;
  write_setup(setup);
END;

PROCEDURE guided_setup(VAR setup: setup_type);

    PROCEDURE guided_pop_size;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                             Population Size');
      writeln;
      writeln('  This parameter lets you set the size of the population on');
      write(' which the');
      writeln(' Genetic Algorithm will perform. With a small population ');
      write(' the');
      writeln(' population runs the risk of being dominated by one fit ');
      write(' chromosome in');
      writeln(' a rapid convergence, however a large population can be ');
      write(' slow to');
      writeln(' execute. For this reason it is best to use a smaller ');
      write(' population if');
      writeln(' Steady State deletion is to be used.');
      REPEAT
        set_window(bottom);
        ClrScr;
        writeln(' Enter value for Population Size');
        writeln(' Minimum = 10, Maximum = 200, in steps of 10');
        writeln(' starting value is 100');
        write(' >');
        read(setup.pop_size)
      UNTIL setup.pop_size MOD 10 = 0;
    END;

    PROCEDURE guided_gen;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                           Number of Generations');
      writeln;
      write('  The Number of Generations over which the Genetic Algorithm ');
      writeln(' performs');
      write(' is a difficult parameter to set correctly. Because the ');
      writeln(' Algorithm');
      write(' acts in a semi-random manner it is impossible to predict ');
      writeln(' in advance');
      writeln(' how quickly a solution will be found.');
      write('  Depending upon the configuration under which the program ');
      writeln('is run, the');
      write(' value you wish to set may vary from ten up to a hundred ');
      writeln(' generations.');
      write('  If Steady State deletion is used a new generation is said ');
      writeln(' to have');
      write(' been produced after the number of new additions to the ');
      writeln(' population is');
      writeln(' equal to the population size.');
      set_window(bottom);
      ClrScr;
      writeln(' Enter value for No. of Generations');
      writeln(' Minimum = 1, Maximum = 200');
      writeln(' starting value is 50');
      write(' >');
      read(setup.gen)
    END;

    PROCEDURE guided_cross_percent;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                           Crossover Percentage');
      writeln;
      write('  The Crossover Percentage is used as a probability of a ');
      writeln('selected');
      write(' chromosome being "mated" with another before addition to ');
      writeln('the new');
      writeln(' population.');
      write('  A high Crossover Percentage will result in genetic ');
      writeln('material being');
      write(' spread quickly but at the expense of a high risk of ');
      writeln('destroying good');
      writeln(' patterns.');
      write('  The Crossover operator used in this program is known as ');
      writeln('"Uniform');
      writeln(' Order-Based Crossover".');
      set_window(bottom);
      ClrScr;
      writeln(' Enter value for Crossover Percentage');
      writeln(' Minimum = 0, Maximum = 100');
      writeln(' Recommended starting value = 60');
      write(' >');
      read(setup.cross_percent)
    END;

    PROCEDURE guided_mut_percent;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                           Mutation Percentage');
      writeln;
      write('  The Mutation Percentage is used as a probability of a ');
      writeln(' selected');
      write(' chromosome being mutated before addition to the new ');
      writeln('population');
      write('  A low Mutation Percentage will result in the algorithm ');
      writeln('converging on');
      write(' a solution more quickly but once convergence has occurred ');
      writeln('the');
      write(' algorithm may find itself stuck at a local maximum. A ');
      writeln('higher rate');
      write(' allows the reintroduction of diversity at the expense of ');
      writeln('risk of' );
      writeln(' destroying good patterns.');
      write('  The Mutation operator used in this program is known as ');
      writeln('"Scrambled ');
      writeln(' Subset Mutation".');
      write('  Note that the value is input in tenths of a percent, i.e. ');
      writeln('10 = 1% .');
      set_window(bottom);
      ClrScr;
      writeln(' Enter value for Mutation Percentage');
      writeln(' Minimum = 0, Maximum = 1000 (tenths of one Percent)');
      writeln(' Recommended starting value = 10');
      write(' >');
      read(setup.mut_percent)
    END;

    PROCEDURE guided_del;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                           Deletion Technique');
      writeln;
      write('  The choice of Deletion Technique is between the following ');
      writeln(':-');
      writeln;
      writeln('  Delete All            - The whole population is replaced');
      write('  Steady State Deletion - The worst two chromosomes are ');
      writeln('replaced by');
      writeln('                          the two new chromosomes.');
      writeln;
      write('  Due to the necessity to sort the population after each ');
      writeln('insertion,');
      write(' the Steady State Deletion technique takes longer per ');
      writeln('generation than');
      write(' Delete All, however this is partially offset due to the ');
      writeln('fact that it');
      writeln(' reaches it''s solution after less generations.');
      REPEAT
        set_window(bottom);
        ClrScr;
        writeln(' Enter required Deletion Technique');
        writeln(' a = delete all , s = steady state deletion');
        writeln(' Recommended starting technique = a');
        write(' >');
        read(setup.del)
      UNTIL (setup.del = 'a') OR (setup.del = 's');
    END;

    PROCEDURE guided_elite;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                              Elitism Value');
      writeln;
      write('  It is possible that the best members of a population may ');
      writeln('fail to ');
      write(' produce children in the next generation. The elitist ');
      writeln('strategy fixes');
      write(' this problem, and potential source of loss of good genetic ');
      writeln('material,');
      write(' by copying the best members of each generation into the ');
      writeln('succeeding');
      writeln(' generation.');
      write('  This parameter allows the setting of the number of ');
      writeln('chromosomes to be');
      writeln(' copied.');
      REPEAT
        set_window(bottom);
        ClrScr;
        writeln(' Enter value for Elitism');
        writeln(' Minimum = 0, Maximum = ',setup.pop_size,', in steps of 2');
        writeln(' Recommended starting value = 2');
        write(' >');
        read(setup.elite)
      UNTIL setup.elite MOD 2 = 0;
    END;

    PROCEDURE guided_fitness;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                            Fitness Technique');
      writeln;
      writeln(' Three Fitness Techniques are supported :-');
      writeln;
      write(' Fitness is Evaluation - The fitness is equal to the ');
      writeln('evaluation.');
      write(' Windowing             - Each chromosome given fitness ');
      writeln('equal to the');
      write('                         amount by which it exceeds the ');
      writeln('minimum');
      writeln('                         evaluation in the population.');
      write(' Linear Normalisation  - The best chromosome is assigned ');
      writeln('a constant');
      write('                         value and then the other ');
      writeln(' chromosomes are');
      write('                         assigned a linearly decreasing ');
      writeln(' value.');
      REPEAT
        set_window(bottom);
        ClrScr;
        writeln(' Enter required Fitness Technique');
        writeln(' e = Evaluation, w = Windowing, l = Linear Norm.');
        writeln(' Recommended starting technique = e');
        write(' >');
        read(setup.fitness)
      UNTIL (setup.fitness = 'e') OR (setup.fitness = 'w')
       OR (setup.fitness = 'l');
    END;

    PROCEDURE guided_guard;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                              Guard Value');
      writeln;
      write('  The Guard is a minimum amount greater than the minimum ');
      writeln('to guard');
      write(' against the lowest chromosomes having no chance at ');
      writeln('reproduction. Any');
      write(' chromosome having a value less than this guard is assigned ');
      writeln('the guard');
      writeln(' value.');
      write('  The higher this value is, the less of an advantage the ');
      writeln('best');
      writeln(' chromosomes have of being selected.');
      set_window(bottom);
      ClrScr;
      writeln(' Enter value for Guard');
      writeln(' Minimum = 0, Maximum = 4580');
      writeln(' Recommended starting value = 1000');
      write(' >');
      read(setup.guard)
    END;

    PROCEDURE guided_dec;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                 Linear Normalisation''s Decrement Value');
      writeln;
      write('  This is the value which will be the difference between ');
      writeln('the fitness');
      writeln(' assigned to each consecutive chromosome.');
      write('  The higher this value is, the greater is the selection ');
      writeln('pressure in');
      writeln(' favour of the highest rated chromosome');
      set_window(bottom);
      ClrScr;
      writeln(' Enter value for Decrement');
      writeln(' Minimum = 0 , Maximum = 9');
      writeln(' no recommended starting value ');
      write(' >');
      read(setup.dec);
    END;

    PROCEDURE guided_best_fitness;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('               Linear Normalisation''s Best fitness Value');
      writeln;
      write('  This is the value which will be assigned to the best ');
      writeln('chromosome. The');
      write(' fitness assigned to the next best chromosome is this value ');
      writeln('minus the');
      writeln(' decrement set on the last screen.');
      write('  The lower this value is, the greater is the selection ');
      writeln('pressure in');
      writeln(' favour of the highest rated chromosome');
      set_window(bottom);
      ClrScr;
      writeln(' Enter value for Best Fitness');
      writeln(' Minimum = ',(setup.dec*setup.pop_size),', Maximum = 10000');
      writeln(' no recommended starting value');
      write(' >');
      read(setup.best_fitness)
    END;

    PROCEDURE guided_results;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                            Results Format');
      writeln;
      write('  The results can be displayed either as a pure evaluation, ');
      writeln('i.e. the');
      write(' distance that the route represents, or as the difference ');
      writeln('between  ');
      writeln(' the evaluation produced and the optimum route. ');
      writeln(' ');
      writeln(' ');
      REPEAT
        set_window(bottom);
        ClrScr;
        writeln(' Enter required Results Format');
        writeln(' p = Pure Fitnesses, d = Difference from optimum');
        writeln(' starting format is p');
        write(' >');
        read(setup.results)
      UNTIL (setup.results = 'p') OR (setup.results = 'd');
    END;

    PROCEDURE guided_print_mode;
    BEGIN
      set_window(middle);
      clrscr;
      writeln;
      writeln('                            Print Mode');
      writeln;
      writeln(' Output can either be to :-');
      writeln;
      writeln(' Screen only ');
      writeln(' Printer and Screen - a hard copy is produced');
      writeln;
      write(' Do not attempt to run with setting p if a printer is not ');
      writeln('available.');
      REPEAT
        set_window(bottom);
        ClrScr;
        writeln(' Enter required Print Mode');
        writeln(' s = screen only, p = printer and screen');
        writeln(' starting mode is s');
        write(' >');
        read(setup.print_mode)
      UNTIL (setup.print_mode = 'p') OR (setup.print_mode = 's');
    END;

BEGIN
  IF setup.config = 'y' THEN
  BEGIN
    guided_pop_size;
    guided_gen;
    guided_results;
    guided_print_mode;
  END;
  guided_cross_percent;
  guided_mut_percent;
  guided_del;
  IF setup.del = 'a' THEN guided_elite;
  guided_fitness;
  IF setup.fitness = 'w' THEN guided_guard;
  IF setup.fitness = 'l' THEN guided_dec;
  IF setup.fitness = 'l' THEN guided_best_fitness;
  write_setup(setup);
END;


PROCEDURE run_screen;

    VAR graphdriver, graphmode : integer;

BEGIN
  setbkcolor(blue);
  setcolor(yellow);
  settextstyle(defaultfont, horizdir, 1);
  outtextxy(64, 32, 'emonstration');
  outtextxy(184, 32, 'f');
  outtextxy(216, 32, 'sing');
  outtextxy(272, 32, 'enetic');
  outtextxy(344, 32, 'lgorithm');
  outtextxy(432, 32, 'earning -   un   ode');
  outtextxy(336, 128, '1.                Evaluation =');
  outtextxy(336, 136, '2.                Evaluation =');
  outtextxy(336, 144, '3.                Evaluation =');
  outtextxy(336, 152, '4.                Evaluation =');
  outtextxy(336, 160, '5.                Evaluation =');
  outtextxy(24, 416, 'Population Size   =');
  outtextxy(24, 424, 'No of Generations =');
  outtextxy(24, 432, 'Crossover         =');
  outtextxy(24, 440, 'Mutation          =');
  outtextxy(24, 448, 'Deletion Tech.    =');
  outtextxy(24, 456, 'Elitism Value     =');
  outtextxy(328, 416, 'Fitness Tech.     =');
  outtextxy(328, 424, 'Guard Value       =');
  outtextxy(328, 432, 'Decrement Value   =');
  outtextxy(328, 440, 'Best Fitness      =');
  outtextxy(328, 448, 'Results Format    =');
  outtextxy(328, 456, 'Print to          =');
  outtextxy(176, 312, 'Diagrammatic');
  outtextxy(176, 320, 'Representation');
  outtextxy(176, 328, 'Of Best');
  outtextxy(176, 336, 'Chromosome');
  outtextxy(344, 312, 'Graphical');
  outtextxy(344, 320, 'Representation');
  outtextxy(344, 328, 'Of Best');
  outtextxy(344, 336, 'Evaluation');
  rectangle(4, 4, 636, 476);
  rectangle(6, 6, 634, 474);
  rectangle(16, 48, 312, 172);
  rectangle(328, 48, 624, 172);
  rectangle(16, 184, 312, 376);
  rectangle(328, 184, 624, 376);
  rectangle(16, 388, 624, 468);
  line(16, 412, 624, 412);
  line(320, 412, 320, 468);
  settextstyle(defaultfont, horizdir, 2);
  outtextxy(48, 25, 'D');
  outtextxy(168, 25, 'O');
  outtextxy(200, 25, 'U');
  outtextxy(256, 25, 'G');
  outtextxy(328, 25, 'A');
  outtextxy(416, 25, 'L');
  outtextxy(512, 25, 'R');
  outtextxy(552, 25, 'M');
  outtextxy(32, 56, 'Currently at');
  outtextxy(32, 72, 'Generation');
  outtextxy(32, 96, 'Average');
  outtextxy(32, 112, 'Best');
  outtextxy(32, 136, 'Chromosome :-');
  outtextxy(336, 56, 'Gen.     Of');
  outtextxy(336, 80, 'Average =');
  outtextxy(336, 104, 'Best Five Chromos');
  outtextxy(124, 392, 'Any key to alter settings');
END;

PROCEDURE show_and_store(results       : char; pop_rec: pop_rec_type;
                         pop_size,
                         gen_no        : integer; print_mode:
char;
                         VAR best_store: best_store_type;
                         VAR best_five : best_five_type);

    VAR total     : longint;
        index, ave: integer;
        temp_ev   : eval_store;
        temp      : string;

BEGIN
  draw_bar(best_store.evals[gen_no-1, 1] , pop_rec.pop_fitness[1, 1], results);
  setcolor(white);
  total := 0;
  FOR index := 1 TO pop_size DO
  BEGIN
    temp_ev := pop_rec.pop_fitness[index];
    total := total + temp_ev[1];
  END;
  ave := total DIV pop_size;
  IF results = 'p' THEN
  BEGIN
    settextstyle(defaultfont, horizdir, 2);
    setcolor(blue);
    str(gen_no-1, temp);
    outtextxy(200, 72, temp);
    setcolor(white);
    str(gen_no, temp);
    outtextxy(200, 72, temp);
    setcolor(blue);
    str(best_store.evals[gen_no-1, 2], temp);
    outtextxy(184, 96, temp);
    setcolor(white);
    str(ave, temp);
    outtextxy(184, 96, temp);
    setcolor(blue);
    str(best_store.evals[gen_no-1, 1], temp);
    outtextxy(184, 112, temp);
    setcolor(white);
    str(pop_rec.pop_fitness[1,1], temp);
    outtextxy(184, 112, temp);
    setcolor(blue);
    outtextxy(32, 152, best_store.chromos[gen_no-1]);
    setcolor(white);
    outtextxy(32, 152, pop_rec.pop[1]);
    settextstyle(defaultfont, horizdir, 1);
    draw_route(best_store.chromos[gen_no-1], blue);
    draw_route(pop_rec.pop[1], white);
    best_store.chromos[gen_no] := pop_rec.pop[1];
    best_store.evals[gen_no, 1] := pop_rec.pop_fitness[1, 1];
    best_store.evals[gen_no, 2] := ave;
    IF gen_no MOD 5 = 0 THEN
    BEGIN
      settextstyle(defaultfont, horizdir, 2);
      setcolor(blue);
      str(gen_no-5, temp);
      outtextxy(416, 56, temp);
      setcolor(white);
      str(gen_no, temp);
      outtextxy(416, 56, temp);
      setcolor(blue);
      str(best_store.evals[gen_no-5, 2], temp);
      outtextxy(496, 80, temp);
      setcolor(white);
      str(ave, temp);
      outtextxy(496, 80, temp);
      settextstyle(defaultfont, horizdir, 1);
      setcolor(blue);
      outtextxy(360, 128, best_five.chromos[1]);
      outtextxy(360, 136, best_five.chromos[2]);
      outtextxy(360, 144, best_five.chromos[3]);
      outtextxy(360, 152, best_five.chromos[4]);
      outtextxy(360, 160, best_five.chromos[5]);
      str(best_five.evals[1, 1], temp);
      outtextxy(584, 128, temp);
      str(best_five.evals[2, 1], temp);
      outtextxy(584, 136, temp);
      str(best_five.evals[3, 1], temp);
      outtextxy(584, 144, temp);
      str(best_five.evals[4, 1], temp);
      outtextxy(584, 152, temp);
      str(best_five.evals[5, 1], temp);
      outtextxy(584, 160, temp);
      FOR index := 1 TO 5 DO
      BEGIN
        best_five.chromos[index] := pop_rec.pop[index];
        best_five.evals[index, 1] := pop_rec.pop_fitness[index, 1]
      END;
      setcolor(white);
      outtextxy(360, 128, best_five.chromos[1]);
      outtextxy(360, 136, best_five.chromos[2]);
      outtextxy(360, 144, best_five.chromos[3]);
      outtextxy(360, 152, best_five.chromos[4]);
      outtextxy(360, 160, best_five.chromos[5]);
      str(best_five.evals[1, 1], temp);
      outtextxy(584, 128, temp);
      str(best_five.evals[2, 1], temp);
      outtextxy(584, 136, temp);
      str(best_five.evals[3, 1], temp);
      outtextxy(584, 144, temp);
      str(best_five.evals[4, 1], temp);
      outtextxy(584, 152, temp);
      str(best_five.evals[5, 1], temp);
      outtextxy(584, 160, temp);
    END;
  END
   ELSE
   BEGIN
     settextstyle(defaultfont, horizdir, 2);
     setcolor(blue);
     str(gen_no-1, temp);
     outtextxy(200, 72, temp);
     setcolor(white);
     str(gen_no, temp);
     outtextxy(200, 72, temp);
     setcolor(blue);
     str(best_store.evals[gen_no-1, 2], temp);
     outtextxy(184, 96, temp);
     setcolor(white);
     str(ave-4580, temp);
     outtextxy(184, 96, temp);
     setcolor(blue);
     str(best_store.evals[gen_no-1, 1], temp);
     outtextxy(184, 112, temp);
     setcolor(white);
     str(pop_rec.pop_fitness[1,1]-4580, temp);
     outtextxy(184, 112, temp);
     setcolor(blue);
     outtextxy(32, 152, best_store.chromos[gen_no-1]);
     setcolor(white);
     outtextxy(32, 152, pop_rec.pop[1]);
     settextstyle(defaultfont, horizdir, 1);
     draw_route(best_store.chromos[gen_no-1], blue);
     draw_route(pop_rec.pop[1], white);
     best_store.chromos[gen_no] := pop_rec.pop[1];
     best_store.evals[gen_no, 1] := pop_rec.pop_fitness[1, 1];
     best_store.evals[gen_no, 2] := ave;
     IF gen_no MOD 5 = 0 THEN
     BEGIN
       settextstyle(defaultfont, horizdir, 2);
       setcolor(blue);
       str(gen_no-5, temp);
       outtextxy(416, 56, temp);
       setcolor(white);
       str(gen_no, temp);
       outtextxy(416, 56, temp);
       setcolor(blue);
       str(best_store.evals[gen_no - 5, 2], temp);
       outtextxy(496, 80, temp);
       setcolor(white);
       str(ave-4580, temp);
       outtextxy(496, 80, temp);
       settextstyle(defaultfont, horizdir, 1);
       setcolor(blue);
       outtextxy(360, 128, best_five.chromos[1]);
       outtextxy(360, 136, best_five.chromos[2]);
       outtextxy(360, 144, best_five.chromos[3]);
       outtextxy(360, 152, best_five.chromos[4]);
       outtextxy(360, 160, best_five.chromos[5]);
       str(best_five.evals[1, 1], temp);
       outtextxy(584, 128, temp);
       str(best_five.evals[2, 1], temp);
       outtextxy(584, 136, temp);
       str(best_five.evals[3, 1], temp);
       outtextxy(584, 144, temp);
       str(best_five.evals[4, 1], temp);
       outtextxy(584, 152, temp);
       str(best_five.evals[5, 1], temp);
       outtextxy(584, 160, temp);
       FOR index := 1 TO 5 DO
       BEGIN
         best_five.chromos[index] := pop_rec.pop[index];
         best_five.evals[index, 1] := pop_rec.pop_fitness[index, 1]-4580
       END;
       setcolor(white);
       outtextxy(360, 128, best_five.chromos[1]);
       outtextxy(360, 136, best_five.chromos[2]);
       outtextxy(360, 144, best_five.chromos[3]);
       outtextxy(360, 152, best_five.chromos[4]);
       outtextxy(360, 160, best_five.chromos[5]);
       str(best_five.evals[1, 1], temp);
       outtextxy(584, 128, temp);
       str(best_five.evals[2, 1], temp);
       outtextxy(584, 136, temp);
       str(best_five.evals[3, 1], temp);
       outtextxy(584, 144, temp);
       str(best_five.evals[4, 1], temp);
       outtextxy(584, 152, temp);
       str(best_five.evals[5, 1], temp);
       outtextxy(584, 160, temp);
     END;
     best_store.chromos[gen_no] := pop_rec.pop[1];
     best_store.evals[gen_no, 1] := pop_rec.pop_fitness[1, 1] - 4580;
     best_store.evals[gen_no, 2] := ave - 4580;
   END;
  IF print_mode = 'p' THEN
  BEGIN
    IF results = 'p' THEN write (lst ,'Generation ',gen_no, ' Average ',
                                 ave, '  Best Eval. ',pop_rec.pop_fitness[1, 1])
     ELSE write (lst, 'Generation ',gen_no, '  Average ', ave - 4580,
                 '  Best Eval. ',pop_rec.pop_fitness[1, 1] - 4580);
    writeln(lst,'   chromo. ', best_store.chromos[gen_no])
  END;
END;

PROCEDURE write_setup(VAR setup: setup_type);

    VAR temp                          : string;
        graphdriver, graphmode, width : integer;

BEGIN
  graphdriver := detect;
  Initgraph(graphdriver , graphmode, '');
  ClearDevice;
  setbkcolor(blue);
  run_screen;
  setup.config :='n';
  setcolor(white);
  settextstyle(defaultfont, horizdir, 2);
  str(setup.gen, temp);
  outtextxy(544, 56, temp);
  settextstyle(defaultfont, horizdir, 1);
  str(setup.pop_size ,temp);
  outtextxy(184, 416, temp);
  str(setup.gen, temp);
  outtextxy(184, 424, temp);
  str(setup.cross_percent, temp);
  outtextxy(184, 432, temp);
  width := (textwidth(temp));
  outtextxy(184 + width, 432, '%');
  str((setup.mut_percent DIV 10), temp);
  outtextxy(184, 440, temp);
  width := (textwidth(temp));
  outtextxy(184 + width, 440, '.');
  width := width + 8;
  str((setup.mut_percent MOD 10), temp);
  outtextxy(184 + width, 440, temp);
  width := width + (textwidth(temp));
  outtextxy(184 + width, 440,'%');
  CASE setup.del OF
   'a' : BEGIN
           outtextxy(184, 448, 'Delete All');
           str(setup.elite, temp);
           outtextxy(184, 456, temp);
         END;
   's' : BEGIN
           outtextxy(184, 448, 'Steady State');
           outtextxy(184, 456, 'N/A');
         END;
  END;
  CASE setup.fitness OF
   'e' : BEGIN
           outtextxy(488, 416, 'Evaluation');
           outtextxy(488, 424, 'N/A');
           outtextxy(488, 432, 'N/A');
           outtextxy(488, 440, 'N/A');
         END;
   'w' : BEGIN
           outtextxy(488, 416, 'Windowing');
           str(setup.guard, temp);
           outtextxy(488, 424, temp);
           outtextxy(488, 432, 'N/A');
           outtextxy(488, 440, 'N/A');
         END;
   'l' : BEGIN
           outtextxy(488, 416, 'Linear Norm.');
           outtextxy(488, 424, 'N/A');
           str(setup.dec, temp);
           outtextxy(488, 432, temp);
           str(setup.best_fitness, temp);
           outtextxy(488, 440, temp);
         END;
  END;
  IF setup.print_mode = 's' THEN outtextxy(488, 456, 'Screen')
   ELSE outtextxy(488, 456, 'Screen & Printer');
  IF setup.results = 'p' THEN outtextxy(488, 448, 'Pure')
   ELSE outtextxy(488, 448, 'Difference');
  IF setup.print_mode = 'p' THEN
  BEGIN
    write(lst, 'Demonstration Of Using Genetic Algorithm Learning');
    writeln(lst, '(c)Brett Parker @ UMIST 1993');
    writeln(lst);
    writeln(lst);
    write(lst, ' Population Size = ', setup.pop_size);
    writeln(lst, '      Number of Generations = ', setup.gen);
    write(lst, ' Crossover = ', setup.cross_percent,'%');
    writeln(lst, '            Mutation = ', (setup.mut_percent DIV 10),
            '.', (setup.mut_percent MOD 10), '%');
    IF setup.del = 's' THEN writeln(lst, ' Steady State Deletion')
    ELSE
     BEGIN
       write(lst, ' Delete All Deletion   ');
       writeln(lst, '     Elitism Value = ', setup.elite);
     END;
    IF setup.fitness = 'e' THEN write(lst, ' Fitness is Evaluation');
    IF setup.fitness = 'w' THEN
    BEGIN
      write(lst, ' Fitness is Windowing ');
      write(lst, '      Guard set at ', setup.guard);
    END;
    IF setup.fitness = 'l' THEN
    BEGIN
      write(lst, ' Linear Normalised Fitness,');
      write(lst, ' Max. Eval. ', setup.best_fitness,',');
      write(lst, ' Dec. by ', setup.dec);
    END;
    writeln(lst);
    writeln(lst);
  END;
END;

PROCEDURE draw_route(chromo: chromo_type; colour: integer);

    VAR index, x1, x2, y1, y2 : integer;

BEGIN
  setcolor(colour);
  FOR index := 1 TO 11 DO
  BEGIN
    CASE chromo[index] OF
     'L' : BEGIN
             x1 := londonx;
             y1 := londony;
           END;
     'A' : BEGIN
             x1 := amsterdamx;
             y1 := amsterdamy;
           END;
     'B' : BEGIN
             x1 := berlinx;
             y1 := berliny;
           END;
     'C' : BEGIN
             x1 := copenhagenx;
             y1 := copenhageny;
           END;
     'M' : BEGIN
             x1 := moscowx;
             y1 := moscowy;
           END;
     'O' : BEGIN
             x1 := oslox;
             y1 := osloy;
           END;
     'P' : BEGIN
             x1 := parisx;
             y1 := parisy;
           END;
     'R' : BEGIN
             x1 := romex;
             y1 := romey;
           END;
     'V' : BEGIN
             x1 := venicex;
             y1 := venicey;
           END;
     'W' : BEGIN
             x1 := warsawx;
             y1 := warsawy;
           END;

     'Z' : BEGIN
             x1 := zurichx;
             y1 := zurichy;
           END;
    END;
    CASE chromo[index+1] OF
     'L' : BEGIN
             x2 := londonx;
             y2 := londony;
           END;
     'A' : BEGIN
             x2 := amsterdamx;
             y2 := amsterdamy;
           END;
     'B' : BEGIN
             x2 := berlinx;
             y2 := berliny;
           END;
     'C' : BEGIN
             x2 := copenhagenx;
             y2 := copenhageny;
           END;
     'M' : BEGIN
             x2 := moscowx;
             y2 := moscowy;
           END;
     'O' : BEGIN
             x2 := oslox;
             y2 := osloy;
           END;
     'P' : BEGIN
             x2 := parisx;
             y2 := parisy;
           END;
     'R' : BEGIN
             x2 := romex;
             y2 := romey;
           END;
     'V' : BEGIN
             x2 := venicex;
             y2 := venicey;
           END;
     'W' : BEGIN
             x2 := warsawx;
             y2 := warsawy;
           END;

     'Z' : BEGIN
             x2 := zurichx;
             y2 := zurichy;
           END;
    END;
    
    line(x1, y1, x2, y2);
  END;
  settextstyle(defaultfont, horizdir, 1);
  setcolor(yellow);
  outtextxy(40, 264, 'L');
  outtextxy(69, 260, 'A');
  outtextxy(110, 267, 'B');
  outtextxy(107, 238, 'C');
  outtextxy(232, 229, 'M');
  outtextxy(107, 200, 'O');
  outtextxy(46, 290, 'P');
  outtextxy(98, 360, 'R');
  outtextxy(101, 331, 'V');
  outtextxy(152, 273, 'W');
  outtextxy(78, 312, 'Z');
END;

PROCEDURE draw_bar(last_eval, eval :integer; results: char);

    VAR height: integer;

BEGIN

  setcolor(yellow);
  settextstyle(defaultfont, horizdir, 1);
  IF results = 'p' THEN
  BEGIN
    outtextxy(460, 205, '4580');
    outtextxy(460, 231, '5000');
    outtextxy(460, 293, '6000');
    outtextxy(460, 356, '7000');
  END
  ELSE
   BEGIN
     outtextxy(460, 205, '   0');
     outtextxy(460, 268, '1000');
     outtextxy(460, 331, '2000');
   END;
  IF ((last_eval > 7000) OR  (last_eval < 4580)) then last_eval := 7000;
  IF ((eval > 7000) OR (eval < 4580)) then eval := 7000;
  setcolor(blue);
  setfillstyle(ltbkslashfill, white);
  height := (7000 - last_eval) DIV 16;
  bar3d(504, 360, 584, 360 - height, 20, false);
  setfillstyle(emptyfill, blue);
  bar3d(504, 209, 584, 360-height, 20, true);
  setcolor(yellow);
  setfillstyle(ltbkslashfill, white);
  height := (7000 - eval) DIV 16;
  bar3d(504, 360, 584, 360 - height, 20, false);
  setfillstyle(emptyfill, yellow);
  bar3d(504, 209, 584, 360 - height, 20, true);
END;

END.